@@ -25,11 +25,13 @@ try:
     import javabridge
     from bioformats.metadatatools import createOMEXMLMetadata
 except ImportError:
-    raise Exception("""Installation of PathML not complete. Please install openjdk8, bioformats, and javabridge:
+    raise Exception(
+        """Installation of PathML not complete. Please install openjdk8, bioformats, and javabridge:
             conda install openjdk==8.0.152
             pip install javabridge==1.0.19 python-bioformats==4.0.0
-            
-            For detailed installation instructions, please see https://github.com/Dana-Farber-AIOS/pathml/""")
+
+            For detailed installation instructions, please see https://github.com/Dana-Farber-AIOS/pathml/"""
+    )
 
 
 class SlideBackend:
@@ -61,6 +63,7 @@ class OpenSlideBackend(SlideBackend):
     def __init__(self, filename):
         self.filename = filename
         self.slide = openslide.open_slide(filename=filename)
+        self.level_count = self.slide.level_count
 
     def __repr__(self):
         return f"OpenSlideBackend('{self.filename}')"
@@ -70,7 +73,7 @@ class OpenSlideBackend(SlideBackend):
         Extract a region of the image
 
         Args:
-            location (Tuple[int, int]): Location of top-left corner of tile
+            location (Tuple[int, int]): Location of top-left corner of tile (i, j)
             size (Union[int, Tuple[int, int]]): Size of each tile. May be a tuple of (height, width) or a
                 single integer, in which case square tiles of that size are generated.
             level (int): level from which to extract chunks. Level 0 is highest resolution.
@@ -95,7 +98,18 @@ class OpenSlideBackend(SlideBackend):
                 level < self.slide.level_count
             ), f"input level {level} invalid for a slide with {self.slide.level_count} levels"
 
-        region = self.slide.read_region(location=location, level=level, size=size)
+        # openslide read_region expects (x, y) coords, so need to switch order for compatibility with pathml (i, j)
+        i, j = location
+
+        # openslide read_region() uses coords in the level 0 reference frame
+        # if we are reading tiles from a higher level, need to convert to level 0 frame by multiplying by scale factor
+        # see: https://github.com/Dana-Farber-AIOS/pathml/issues/240
+        coord_scale_factor = int(self.slide.level_downsamples[level])
+        i *= coord_scale_factor
+        j *= coord_scale_factor
+
+        h, w = size
+        region = self.slide.read_region(location=(j, i), level=level, size=(w, h))
         region_rgb = pil_to_rgb(region)
         return region_rgb
 
@@ -107,7 +121,7 @@ class OpenSlideBackend(SlideBackend):
             level (int): Which level to get shape from. Level 0 is highest resolution. Defaults to 0.
 
         Returns:
-            Tuple[int, int]: Shape of image at target level.
+            Tuple[int, int]: Shape of image at target level, in (i, j) coordinates.
         """
         assert isinstance(level, int), f"level {level} invalid. Must be an int."
         assert (
@@ -228,7 +242,7 @@ class BioFormatsBackend(SlideBackend):
     """
     Use BioFormats to interface with image files.
 
-    Does not support multi-level images.
+    Now support multi-level images.
     Depends on `python-bioformats <https://github.com/CellProfiler/python-bioformats>`_ which wraps ome bioformats
     java library, parses pixel and metadata of proprietary formats, and
     converts all formats to OME-TIFF. Please cite: https://pubmed.ncbi.nlm.nih.gov/20513764/
@@ -248,33 +262,55 @@ class BioFormatsBackend(SlideBackend):
         omeMeta = createOMEXMLMetadata()
         reader.setMetadataStore(omeMeta)
         reader.setId(str(self.filename))
-        sizex, sizey, sizez, sizec, sizet = (
-            reader.getSizeX(),
-            reader.getSizeY(),
-            reader.getSizeZ(),
-            reader.getSizeC(),
-            reader.getSizeT(),
-        )
-        self.shape = (sizex, sizey, sizez, sizec, sizet)
-        self.imagecache = None
+        seriesCount = reader.getSeriesCount()
+
+        sizeSeries = []
+        for s in range(seriesCount):
+            reader.setSeries(s)
+            sizex, sizey, sizez, sizec, sizet = (
+                reader.getSizeX(),
+                reader.getSizeY(),
+                reader.getSizeZ(),
+                reader.getSizeC(),
+                reader.getSizeT(),
+            )
+            sizeSeries.append((sizex, sizey, sizez, sizec, sizet))
+        s = [s[0] * s[1] for s in sizeSeries]
+
+        self.level_count = seriesCount  # count of levels
+        self.shape = sizeSeries[s.index(max(s))]  # largest shape
+        self.shape_list = sizeSeries  # shape on all levels
         self.metadata = bioformats.get_omexml_metadata(self.filename)
 
     def __repr__(self):
         return f"BioFormatsBackend('{self.filename}')"
 
-    def get_image_shape(self):
+    def get_image_shape(self, level=None):
         """
-        Get the shape of the image.
+        Get the shape of the image on specific level.
+
+        Args:
+            level (int): Which level to get shape from. If ``level is None``, returns the shape of the biggest level.
+                Defaults to ``None``.
 
         Returns:
             Tuple[int, int]: Shape of image (H, W)
         """
-        return self.shape[:2]
+        if level is None:
+            return self.shape[:2]
+        else:
+            assert isinstance(level, int), f"level {level} invalid. Must be an int."
+            assert (
+                level < self.level_count
+            ), f"input level {level} invalid for slide with {self.level_count} levels total"
+            return self.shape_list[level][:2]
 
-    def extract_region(self, location, size, level=None):
+    def extract_region(self, location, size, level=0):
         """
         Extract a region of the image. All bioformats images have 5 dimensions representing
-        (x, y, z, channel, time). If a tuple with len < 5 is passed, missing dimensions will be
+        (x, y, z, channel, time). Even if an image does not have multiple z-series or time-series,
+        those dimensions will still be kept. For example, a standard RGB image will be of shape (x, y, 1, 3, 1).
+        If a tuple with len < 5 is passed, missing dimensions will be
         retrieved in full.
 
         Args:
@@ -282,6 +318,7 @@ class BioFormatsBackend(SlideBackend):
             size (Tuple[int, int, ...]): (X,Y) size of each region. If an integer is passed, will convert to a
             tuple of (H, W) and extract a square region. If a tuple with len < 5 is passed, missing
                 dimensions will be retrieved in full.
+            level (int): level from which to extract chunks. Level 0 is highest resolution.
 
         Returns:
             np.ndarray: image at the specified region
@@ -290,10 +327,14 @@ class BioFormatsBackend(SlideBackend):
             Extract 2000x2000 x,y region from upper left corner of 7 channel, 2d fluorescent image.
             data.slide.extract_region(location = (0,0), size = 2000)
         """
-        if level not in [None, 0]:
-            raise ValueError(
-                "BioFormatsBackend does not support levels, please pass a level in [None, 0]"
-            )
+        if level is None:
+            level = 0
+        else:
+            assert isinstance(level, int), f"level {level} invalid. Must be an int."
+            assert (
+                level < self.level_count
+            ), f"input level {level} invalid for slide with {self.level_count} levels total"
+
         # if a single int is passed for size, convert to a tuple to get a square region
         if type(size) is int:
             size = (size, size)
@@ -314,27 +355,48 @@ class BioFormatsBackend(SlideBackend):
                 f"input size {size} invalid. Must be a tuple of integer coordinates of len<2"
             )
         javabridge.start_vm(class_path=bioformats.JARS, max_heap_size="100G")
-        reader = bioformats.ImageReader(str(self.filename), perform_init=True)
-        # expand size
-        size = list(size)
-        arrayshape = list(size)
-        for i in range(len(self.shape)):
-            if i > len(size) - 1:
-                arrayshape.append(self.shape[i])
-        arrayshape = tuple(arrayshape)
-        array = np.empty(arrayshape)
-        sample = reader.read(
-            z=0, t=0, rescale=False, XYWH=(location[0], location[1], size[0], size[1])
-        )
-        # if series is set to read only one channel, explicitly read c
-        if len(sample.shape) == 2:
-            for z in range(self.shape[2]):
-                for c in range(self.shape[3]):
-                    for t in range(self.shape[4]):
+        with bioformats.ImageReader(str(self.filename), perform_init=True) as reader:
+            # expand size
+            size = list(size)
+            arrayshape = list(size)
+            for i in range(len(self.shape_list[level])):
+                if i > len(size) - 1:
+                    arrayshape.append(self.shape_list[level][i])
+            arrayshape = tuple(arrayshape)
+            array = np.empty(arrayshape)
+
+            sample = reader.read(
+                z=0,
+                t=0,
+                series=level,
+                rescale=False,
+                XYWH=(location[0], location[1], size[0], size[1]),
+            )
+
+            if len(sample.shape) == 2:
+                for z in range(self.shape_list[level][2]):
+                    for c in range(self.shape_list[level][3]):
+                        for t in range(self.shape_list[level][4]):
+                            slicearray = reader.read(
+                                z=z,
+                                t=t,
+                                series=level,
+                                rescale=False,
+                                XYWH=(location[0], location[1], size[0], size[1]),
+                            )
+                            slicearray = np.asarray(slicearray)
+                            # some file formats read x, y out of order, transpose
+                            if slicearray.shape[:2] != array.shape[:2]:
+                                slicearray = np.transpose(slicearray)
+                            array[:, :, z, c, t] = slicearray
+            # if series is set to read all channels, read all c simultaneously
+            elif len(sample.shape) == 3:
+                for z in range(self.shape_list[level][2]):
+                    for t in range(self.shape_list[level][4]):
                         slicearray = reader.read(
                             z=z,
                             t=t,
-                            series=c,
+                            series=level,
                             rescale=False,
                             XYWH=(location[0], location[1], size[0], size[1]),
                         )
@@ -342,25 +404,11 @@ class BioFormatsBackend(SlideBackend):
                         # some file formats read x, y out of order, transpose
                         if slicearray.shape[:2] != array.shape[:2]:
                             slicearray = np.transpose(slicearray)
-                        array[:, :, z, c, t] = slicearray
-        # if series is set to read all channels, read all c simultaneously
-        elif len(sample.shape) == 3:
-            for z in range(self.shape[2]):
-                for t in range(self.shape[4]):
-                    slicearray = reader.read(
-                        z=z,
-                        t=t,
-                        rescale=False,
-                        XYWH=(location[0], location[1], size[0], size[1]),
-                    )
-                    slicearray = np.asarray(slicearray)
-                    # some file formats read x, y out of order, transpose
-                    if slicearray.shape[:2] != array.shape[:2]:
-                        slicearray = np.transpose(slicearray)
-                        slicearray = np.moveaxis(slicearray, 0, -1)
-                    array[:, :, z, :, t] = slicearray
-        else:
-            raise Exception("image format not supported")
+                            slicearray = np.moveaxis(slicearray, 0, -1)
+                        array[:, :, z, :, t] = slicearray
+            else:
+                raise Exception("image format not supported")
+
         array = array.astype(np.uint8)
         return array
 
@@ -371,10 +419,8 @@ class BioFormatsBackend(SlideBackend):
 
         Args:
             size (Tuple[int, int]): thumbnail size
-
         Returns:
             np.ndarray: RGB thumbnail image
-
         Example:
             Get 1000x1000 thumbnail of 7 channel fluorescent image.
             shape = data.slide.get_image_shape()
@@ -422,7 +468,10 @@ class BioFormatsBackend(SlideBackend):
         Yields:
             pathml.core.tile.Tile: Extracted Tile object
         """
-        assert level == 0 or level is None, f"bioformats does not support levels"
+        assert isinstance(level, int), f"level {level} invalid. Must be an int."
+        assert (
+            level < self.level_count
+        ), f"input level {level} invalid for slide with {self.level_count} levels total"
         assert isinstance(shape, int) or (
             isinstance(shape, tuple) and len(shape) == 2
         ), f"input shape {shape} invalid. Must be a tuple of (H, W), or a single integer for square tiles"
@@ -439,7 +488,7 @@ class BioFormatsBackend(SlideBackend):
         elif isinstance(stride, int):
             stride = (stride, stride)
 
-        i, j = self.get_image_shape()
+        i, j = self.get_image_shape(level=level)
 
         stride_i, stride_j = stride
 
@@ -456,14 +505,18 @@ class BioFormatsBackend(SlideBackend):
                 coords = (int(ix_i * stride_i), int(ix_j * stride_j))
                 if coords[0] + shape[0] < i and coords[1] + shape[1] < j:
                     # get image for tile
-                    tile_im = self.extract_region(location=coords, size=shape)
+                    tile_im = self.extract_region(
+                        location=coords, size=shape, level=level
+                    )
                     yield pathml.core.tile.Tile(image=tile_im, coords=coords)
                 else:
                     unpaddedshape = (
                         i - coords[0] if coords[0] + shape[0] > i else shape[0],
                         j - coords[1] if coords[1] + shape[1] > j else shape[1],
                     )
-                    tile_im = self.extract_region(location=coords, size=unpaddedshape)
+                    tile_im = self.extract_region(
+                        location=coords, size=unpaddedshape, level=level
+                    )
                     zeroarrayshape = list(tile_im.shape)
                     zeroarrayshape[0], zeroarrayshape[1] = (
                         list(shape)[0],
